package maze.searchers;

import static org.junit.Assert.*;

import java.util.HashSet;
import java.util.Random;
import java.util.function.Supplier;


public class SearcherTest {

	public static void testRetrieval(Supplier<Searcher<Integer>> maker) {
		Searcher<Integer> searcher = maker.get();
		assertTrue(searcher.isEmpty());

		Random r = new Random();
		for (int round = 1; round <= 10; round++) {
			HashSet<Integer> wentIn = new HashSet<>();
			int limit = 100 + r.nextInt(10000);
			for (int i = 0; i < limit; i++) {
				wentIn.add(i);
				searcher.add(i);
			}
			assertFalse(searcher.isEmpty());

			while (!searcher.isEmpty()) {
				int peeked = searcher.peek();
				assertTrue(wentIn.contains(peeked));
				int oldSize = searcher.size();
				int value = searcher.remove();
				assertEquals(oldSize - 1, searcher.size());
				assertEquals(peeked, value);
				wentIn.remove(value);
			}
		}
	}

	public static void testCapacity(Supplier<Searcher<Integer>> maker) {
		Searcher<Integer> searcher = maker.get();
		int capacityTest = 100;
		for (int i = 0; i < 10; i++) {
			for (int j = 0; j < capacityTest; j++) {
				searcher.add(j);
			}
			while (!searcher.isEmpty()) {
				searcher.remove();
			}
			assertTrue(searcher.capacity() <= 2 * capacityTest);
		}
	}
}
